@@ -1,6 +1,8 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Batch acquisition functions using the reparameterization trick in combination
@@ -16,6 +18,8 @@ with (quasi) Monte-Carlo sampling. See [Rezende2014reparam]_ and
     The reparameterization trick for acquisition functions. ArXiv 2017.
 """
 
+from __future__ import annotations
+
 import math
 from abc import ABC, abstractmethod
 from typing import Optional, Union
@@ -33,6 +37,7 @@ from ..utils.transforms import (
 )
 from .acquisition import AcquisitionFunction
 from .objective import IdentityMCObjective, MCAcquisitionObjective
+from .utils import prune_inferior_points
 
 
 class MCAcquisitionFunction(AcquisitionFunction, ABC):
@@ -62,6 +67,10 @@ class MCAcquisitionFunction(AcquisitionFunction, ABC):
             sampler = SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)
         self.add_module("sampler", sampler)
         if objective is None:
+            if model.num_outputs != 1:
+                raise UnsupportedError(
+                    "Must specify an objective when using a multi-output model."
+                )
             objective = IdentityMCObjective()
         elif not isinstance(objective, MCAcquisitionObjective):
             raise UnsupportedError(
@@ -127,6 +136,11 @@ class qExpectedImprovement(MCAcquisitionFunction):
         )
         if not torch.is_tensor(best_f):
             best_f = torch.tensor(float(best_f))
+        elif not (best_f.ndim < 2 and best_f.numel() == 1):
+            raise ValueError(
+                "best_f must be either a scalar or a single-elment "
+                "one-dimensional tensor."
+            )
         self.register_buffer("best_f", best_f)
 
     @concatenate_pending_points
@@ -175,6 +189,7 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
         X_pending: Optional[Tensor] = None,
+        prune_baseline: bool = False,
     ) -> None:
         r"""q-Noisy Expected Improvement.
 
@@ -191,10 +206,20 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
                 points that have been submitted for function evaluation
                 but have not yet been evaluated.  Concatenated into X upon
                 forward call.  Copied and set to have no gradient.
+            prune_baseline: If True, remove points in `X_baseline` that are
+                highly unlikely to be the best point. This can significantly
+                improve performance and is generally recommended. In order to
+                customize pruning parameters, instead manually call
+                `botorch.acquisition.utils.prune_inferior_points` on `X_baseline`
+                before instantiating the acquisition function.
         """
         super().__init__(
             model=model, sampler=sampler, objective=objective, X_pending=X_pending
         )
+        if prune_baseline:
+            X_baseline = prune_inferior_points(
+                model=model, X=X_baseline, objective=objective
+            )
         self.register_buffer("X_baseline", X_baseline)
 
     @concatenate_pending_points
