@@ -1,11 +1,15 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Candidate generation utilities.
 """
 
+from __future__ import annotations
+
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 import torch
@@ -209,7 +213,6 @@ def gen_candidates_torch(
 
         def closure():
             bayes_optimizer.zero_grad()
-            loss = -acquisition_function(candidates).sum()
             loss.backward()
             return loss
 
