@@ -1,12 +1,16 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Gaussian Process Regression models based on GPyTorch models.
 """
 
-from typing import Any, Optional
+from __future__ import annotations
+
+from typing import Any, List, Optional, Union
 
 import torch
 from gpytorch.constraints.constraints import GreaterThan
@@ -21,14 +25,17 @@ from gpytorch.likelihoods.gaussian_likelihood import (
 from gpytorch.likelihoods.likelihood import Likelihood
 from gpytorch.likelihoods.noise_models import HeteroskedasticNoise
 from gpytorch.means.constant_mean import ConstantMean
+from gpytorch.mlls.noise_model_added_loss_term import NoiseModelAddedLossTerm
 from gpytorch.models.exact_gp import ExactGP
 from gpytorch.module import Module
 from gpytorch.priors.smoothed_box_prior import SmoothedBoxPrior
 from gpytorch.priors.torch_priors import GammaPrior
 from torch import Tensor
 
+from .. import settings
 from ..sampling.samplers import MCSampler
 from .gpytorch import BatchedMultiOutputGPyTorchModel
+from .transforms.outcome import Log, OutcomeTransform
 from .utils import validate_input_scaling
 
 
@@ -46,10 +53,10 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
     When the training observations include multiple outputs, this model will use
     batching to model outputs independently.
 
-    Use this model when you have independent output(s) and all outputs use the same
-    training data. If outputs are independent and outputs have different training
-    data, use the ModelListGP. When modeling correlations between outputs, use
-    the MultiTaskGP.
+    Use this model when you have independent output(s) and all outputs use the
+    same training data. If outputs are independent and outputs have different
+    training data, use the ModelListGP. When modeling correlations between
+    outputs, use the MultiTaskGP.
     """
 
     def __init__(
@@ -58,26 +65,31 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         train_Y: Tensor,
         likelihood: Optional[Likelihood] = None,
         covar_module: Optional[Module] = None,
+        outcome_transform: Optional[OutcomeTransform] = None,
     ) -> None:
         r"""A single-task exact GP model.
 
         Args:
-            train_X: A `n x d` or `batch_shape x n x d` (batch mode) tensor of training
-                features.
-            train_Y: A `n x m` or `batch_shape x n x m` (batch mode) tensor of
-                training observations.
+            train_X: A `batch_shape x n x d` tensor of training features.
+            train_Y: A `batch_shape x n x m` tensor of training observations.
             likelihood: A likelihood. If omitted, use a standard
                 GaussianLikelihood with inferred noise level.
-            covar_module: The covariance (kernel) matrix. If omitted, use the
-                MaternKernel.
+            covar_module: The module computing the covariance (Kernel) matrix.
+                If omitted, use a `MaternKernel`.
+            outcome_transform: An outcome transform that is applied to the
+                training data during instantiation and to the posterior during
+                inference (that is, the `Posterior` obtained by calling
+                `.posterior` on the model will be on the original scale).
 
         Example:
             >>> train_X = torch.rand(20, 2)
             >>> train_Y = torch.sin(train_X).sum(dim=1, keepdim=True)
             >>> model = SingleTaskGP(train_X, train_Y)
         """
-        validate_input_scaling(train_X=train_X, train_Y=train_Y)
+        if outcome_transform is not None:
+            train_Y, _ = outcome_transform(train_Y)
         self._validate_tensor_args(X=train_X, Y=train_Y)
+        validate_input_scaling(train_X=train_X, train_Y=train_Y)
         self._set_dimensions(train_X=train_X, train_Y=train_Y)
         train_X, train_Y, _ = self._transform_tensor_args(X=train_X, Y=train_Y)
         if likelihood is None:
@@ -107,8 +119,17 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
                 batch_shape=self._aug_batch_shape,
                 outputscale_prior=GammaPrior(2.0, 0.15),
             )
+            self._subset_batch_dict = {
+                "likelihood.noise_covar.raw_noise": -2,
+                "mean_module.constant": -2,
+                "covar_module.raw_outputscale": -1,
+                "covar_module.base_kernel.raw_lengthscale": -3,
+            }
         else:
             self.covar_module = covar_module
+        # TODO: Allow subsetting of other covar modules
+        if outcome_transform is not None:
+            self.outcome_transform = outcome_transform
         self.to(train_X)
 
     def forward(self, x: Tensor) -> MultivariateNormal:
@@ -128,16 +149,25 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
     This model works in batch mode (each batch having its own hyperparameters).
     """
 
-    def __init__(self, train_X: Tensor, train_Y: Tensor, train_Yvar: Tensor) -> None:
+    def __init__(
+        self,
+        train_X: Tensor,
+        train_Y: Tensor,
+        train_Yvar: Tensor,
+        covar_module: Optional[Module] = None,
+        outcome_transform: Optional[OutcomeTransform] = None,
+    ) -> None:
         r"""A single-task exact GP model using fixed noise levels.
 
         Args:
-            train_X: A `n x d` or `batch_shape x n x d` (batch mode) tensor of training
-                features.
-            train_Y: A `n x m` or `batch_shape x n x m` (batch mode) tensor of
-                training observations.
-            train_Yvar: A `batch_shape x n x m` or `batch_shape x n x m`
-                (batch mode) tensor of observed measurement noise.
+            train_X: A `batch_shape x n x d` tensor of training features.
+            train_Y: A `batch_shape x n x m` tensor of training observations.
+            train_Yvar: A `batch_shape x n x m` tensor of observed measurement
+                noise.
+            outcome_transform: An outcome transform that is applied to the
+                training data during instantiation and to the posterior during
+                inference (that is, the `Posterior` obtained by calling
+                `.posterior` on the model will be on the original scale).
 
         Example:
             >>> train_X = torch.rand(20, 2)
@@ -145,8 +175,10 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             >>> train_Yvar = torch.full_like(train_Y, 0.2)
             >>> model = FixedNoiseGP(train_X, train_Y, train_Yvar)
         """
-        validate_input_scaling(train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
+        if outcome_transform is not None:
+            train_Y, train_Yvar = outcome_transform(train_Y, train_Yvar)
         self._validate_tensor_args(X=train_X, Y=train_Y, Yvar=train_Yvar)
+        validate_input_scaling(train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
         self._set_dimensions(train_X=train_X, train_Y=train_Y)
         train_X, train_Y, train_Yvar = self._transform_tensor_args(
             X=train_X, Y=train_Y, Yvar=train_Yvar
@@ -158,31 +190,44 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             self, train_inputs=train_X, train_targets=train_Y, likelihood=likelihood
         )
         self.mean_module = ConstantMean(batch_shape=self._aug_batch_shape)
-        self.covar_module = ScaleKernel(
-            base_kernel=MaternKernel(
-                nu=2.5,
-                ard_num_dims=train_X.shape[-1],
+        if covar_module is None:
+            self.covar_module = ScaleKernel(
+                base_kernel=MaternKernel(
+                    nu=2.5,
+                    ard_num_dims=train_X.shape[-1],
+                    batch_shape=self._aug_batch_shape,
+                    lengthscale_prior=GammaPrior(3.0, 6.0),
+                ),
                 batch_shape=self._aug_batch_shape,
-                lengthscale_prior=GammaPrior(3.0, 6.0),
-            ),
-            batch_shape=self._aug_batch_shape,
-            outputscale_prior=GammaPrior(2.0, 0.15),
-        )
+                outputscale_prior=GammaPrior(2.0, 0.15),
+            )
+            self._subset_batch_dict = {
+                "mean_module.constant": -2,
+                "covar_module.raw_outputscale": -1,
+                "covar_module.base_kernel.raw_lengthscale": -3,
+            }
+        else:
+            self.covar_module = covar_module
+        # TODO: Allow subsetting of other covar modules
+        if outcome_transform is not None:
+            self.outcome_transform = outcome_transform
+
         self.to(train_X)
 
     def fantasize(
         self,
         X: Tensor,
         sampler: MCSampler,
-        observation_noise: bool = True,
+        observation_noise: Union[bool, Tensor] = True,
         **kwargs: Any,
-    ) -> "FixedNoiseGP":
+    ) -> FixedNoiseGP:
         r"""Construct a fantasy model.
 
         Constructs a fantasy model in the following fashion:
         (1) compute the model posterior at `X` (if `observation_noise=True`,
-        this includes observation noise, which is taken as the mean across
-        the observation noise in the training data).
+        this includes observation noise taken as the mean across the observation
+        noise in the training data. If `observation_noise` is a Tensor, use
+        it directly as the observation noise to add).
         (2) sample from this posterior (using `sampler`) to generate "fake"
         observations.
         (3) condition the model on the new fake observations.
@@ -195,12 +240,15 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             sampler: The sampler used for sampling from the posterior at `X`.
             observation_noise: If True, include the mean across the observation
                 noise in the training data as observation noise in the posterior
-                from which the samples are drawn.
+                from which the samples are drawn. If a Tensor, use it directly
+                as the specified measurement noise.
 
         Returns:
             The constructed fantasy model.
         """
-        post_X = self.posterior(X, observation_noise=observation_noise, **kwargs)
+        propagate_grads = kwargs.pop("propagate_grads", False)
+        with settings.propagate_grads(propagate_grads):
+            post_X = self.posterior(X, observation_noise=observation_noise, **kwargs)
         Y_fantasized = sampler(post_X)  # num_fantasies x batch_shape x n' x m
         # Use the mean of the previous noise values (TODO: be smarter here).
         # noise should be batch_shape x q x m when X is batch_shape x q x d, and
@@ -214,25 +262,50 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         covar_x = self.covar_module(x)
         return MultivariateNormal(mean_x, covar_x)
 
+    def subset_output(self, idcs: List[int]) -> BatchedMultiOutputGPyTorchModel:
+        r"""Subset the model along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the model to.
+
+        Returns:
+            The current model, subset to the specified output indices.
+        """
+        new_model = super().subset_output(idcs=idcs)
+        full_noise = new_model.likelihood.noise_covar.noise
+        new_noise = full_noise[..., idcs if len(idcs) > 1 else idcs[0], :]
+        new_model.likelihood.noise_covar.noise = new_noise
+        return new_model
+
 
 class HeteroskedasticSingleTaskGP(SingleTaskGP):
     r"""A single-task exact GP model using a heteroskeastic noise model.
 
-    This model internally wraps another GP (a SingleTaskGP) to model the observation
-    noise. This allows the likelihood to make out-of-sample predictions for the
-    observation noise levels.
+    This model internally wraps another GP (a SingleTaskGP) to model the
+    observation noise. This allows the likelihood to make out-of-sample
+    predictions for the observation noise levels.
     """
 
-    def __init__(self, train_X: Tensor, train_Y: Tensor, train_Yvar: Tensor) -> None:
+    def __init__(
+        self,
+        train_X: Tensor,
+        train_Y: Tensor,
+        train_Yvar: Tensor,
+        outcome_transform: Optional[OutcomeTransform] = None,
+    ) -> None:
         r"""A single-task exact GP model using a heteroskedastic noise model.
 
         Args:
-            train_X: A `n x d` or `batch_shape x n x d` (batch mode) tensor of training
-                features.
-            train_Y: A `n x m` or `batch_shape x n x m` (batch mode) tensor of
-                training observations.
-            train_Yvar: A `batch_shape x n x m` or `batch_shape x n x m`
-                (batch mode) tensor of observed measurement noise.
+            train_X: A `batch_shape x n x d` tensor of training features.
+            train_Y: A `batch_shape x n x m` tensor of training observations.
+            train_Yvar: A `batch_shape x n x m` tensor of observed measurement
+                noise.
+            outcome_transform: An outcome transform that is applied to the
+                training data during instantiation and to the posterior during
+                inference (that is, the `Posterior` obtained by calling
+                `.posterior` on the model will be on the original scale).
+                Note that the noise model internally log-transforms the
+                variances, which will happen after this transform is applied.
 
         Example:
             >>> train_X = torch.rand(20, 2)
@@ -241,8 +314,10 @@ class HeteroskedasticSingleTaskGP(SingleTaskGP):
             >>> train_Yvar = 0.1 + se * torch.rand_like(train_Y)
             >>> model = HeteroskedasticSingleTaskGP(train_X, train_Y, train_Yvar)
         """
-        validate_input_scaling(train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
+        if outcome_transform is not None:
+            train_Y, train_Yvar = outcome_transform(train_Y, train_Yvar)
         self._validate_tensor_args(X=train_X, Y=train_Y, Yvar=train_Yvar)
+        validate_input_scaling(train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
         self._set_dimensions(train_X=train_X, train_Y=train_Y)
         noise_likelihood = GaussianLikelihood(
             noise_prior=SmoothedBoxPrior(-3, 5, 0.5, transform=torch.log),
@@ -252,14 +327,25 @@ class HeteroskedasticSingleTaskGP(SingleTaskGP):
             ),
         )
         noise_model = SingleTaskGP(
-            train_X=train_X, train_Y=train_Yvar.log(), likelihood=noise_likelihood
+            train_X=train_X,
+            train_Y=train_Yvar,
+            likelihood=noise_likelihood,
+            outcome_transform=Log(),
         )
-
         likelihood = _GaussianLikelihoodBase(HeteroskedasticNoise(noise_model))
         super().__init__(train_X=train_X, train_Y=train_Y, likelihood=likelihood)
+        self.register_added_loss_term("noise_added_loss")
+        self.update_added_loss_term(
+            "noise_added_loss", NoiseModelAddedLossTerm(noise_model)
+        )
+        if outcome_transform is not None:
+            self.outcome_transform = outcome_transform
         self.to(train_X)
 
     def condition_on_observations(
         self, X: Tensor, Y: Tensor, **kwargs: Any
-    ) -> "HeteroskedasticSingleTaskGP":
+    ) -> HeteroskedasticSingleTaskGP:
+        raise NotImplementedError
+
+    def subset_output(self, idcs: List[int]) -> HeteroskedasticSingleTaskGP:
         raise NotImplementedError
