@@ -1,6 +1,8 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Abstract model class for all GPyTorch-based botorch models.
@@ -9,24 +11,31 @@ To implement your own, simply inherit from both the provided classes and a
 GPyTorch Model class such as an ExactGP.
 """
 
+from __future__ import annotations
+
+import itertools
 import warnings
-from abc import ABC, abstractproperty
-from contextlib import ExitStack
-from typing import Any, List, Optional, Tuple
+from abc import ABC
+from copy import deepcopy
+from typing import Any, Iterator, List, Optional, Tuple, Union
 
 import torch
-from gpytorch import settings as gpt_settings
 from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
 from gpytorch.lazy import lazify
 from gpytorch.likelihoods.gaussian_likelihood import FixedNoiseGaussianLikelihood
 from torch import Tensor
 
-from .. import settings
 from ..exceptions.errors import BotorchTensorDimensionError
 from ..exceptions.warnings import BotorchTensorDimensionWarning
 from ..posteriors.gpytorch import GPyTorchPosterior
+from ..utils.transforms import gpt_posterior_settings
 from .model import Model
-from .utils import _make_X_full, add_output_dim, multioutput_to_batch_mode_transform
+from .utils import (
+    _make_X_full,
+    add_output_dim,
+    mod_batch_shape,
+    multioutput_to_batch_mode_transform,
+)
 
 
 class GPyTorchModel(Model, ABC):
@@ -47,17 +56,17 @@ class GPyTorchModel(Model, ABC):
         the same `batch_shape`.
 
         Args:
-            X: A `n x d` or `batch_shape x n x d`-dim Tensor, where `d` is the
-                dimension of the feature space, `n` is the number of points per
-                batch, and `batch_shape` is the batch shape.
-            Y: A `n x m` or `batch_shape x n x m`-dim Tensor, where `m` is the
-                number of model outputs, `n'` is the number of points per batch,
-                and `batch_shape'` is the batch shape of the observations.
-            Yvar: A `n x m` or `batch_shape x n x m` (batch mode) tensor of
-                observed measurement noise. Note: this will be None when using a
-                model that infers the noise level (e.g. a `SingleTaskGP`).
-            strict: A boolean indicating whether to check that `Y` and `Yvar` have
-                an explicit output dimension.
+            X: A `batch_shape x n x d`-dim Tensor, where `d` is the dimension of
+                the feature space, `n` is the number of points per batch, and
+                `batch_shape` is the batch shape (potentially empty).
+            Y: A `batch_shape' x n x m`-dim Tensor, where `m` is the number of
+                model outputs, `n'` is the number of points per batch, and
+                `batch_shape'` is the batch shape of the observations.
+            Yvar: A `batch_shape' x n x m` tensor of observed measurement noise.
+                Note: this will be None when using a model that infers the noise
+                level (e.g. a `SingleTaskGP`).
+            strict: A boolean indicating whether to check that `Y` and `Yvar`
+                have an explicit output dimension.
         """
         if strict:
             if X.dim() != Y.dim():
@@ -68,7 +77,7 @@ class GPyTorchModel(Model, ABC):
                     )
                 else:
                     message = (
-                        f"Expected X and Y to have the same number of dimensions"
+                        "Expected X and Y to have the same number of dimensions"
                         f" (got X with dimension {X.dim()} and Y with dimension"
                         f" {Y.dim()}."
                     )
@@ -89,35 +98,47 @@ class GPyTorchModel(Model, ABC):
                 f" {Yvar.shape})."
             )
 
+    @property
+    def num_outputs(self) -> int:
+        r"""The number of outputs of the model."""
+        return self._num_outputs
+
     def posterior(
-        self, X: Tensor, observation_noise: bool = False, **kwargs: Any
+        self, X: Tensor, observation_noise: Union[bool, Tensor] = False, **kwargs: Any
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
 
         Args:
-            X: A `(batch_shape) x q x d`-dim Tensor, where `d` is the dimension of the
-                feature space and `q` is the number of points considered jointly.
-            observation_noise: If True, add observation noise to the posterior.
+            X: A `(batch_shape) x q x d`-dim Tensor, where `d` is the dimension
+                of the feature space and `q` is the number of points considered
+                jointly.
+            observation_noise: If True, add the observation noise from the
+                likelihood to the posterior. If a Tensor, use it directly as the
+                observation noise (must be of shape `(batch_shape) x q`).
 
         Returns:
             A `GPyTorchPosterior` object, representing a batch of `b` joint
             distributions over `q` points. Includes observation noise if
-            `observation_noise=True`.
+            specified.
         """
         self.eval()  # make sure model is in eval mode
-        with ExitStack() as es:
-            es.enter_context(gpt_settings.debug(False))
-            es.enter_context(gpt_settings.fast_pred_var())
-            es.enter_context(
-                gpt_settings.detach_test_caches(settings.propagate_grads.off())
-            )
+        with gpt_posterior_settings():
             mvn = self(X)
-            if observation_noise:
-                # TODO: Allow passing in observation noise via kwarg
-                mvn = self.likelihood(mvn, X)
-            return GPyTorchPosterior(mvn=mvn)
+            if observation_noise is not False:
+                if torch.is_tensor(observation_noise):
+                    # TODO: Make sure observation noise is transformed correctly
+                    self._validate_tensor_args(X=X, Y=observation_noise)
+                    if observation_noise.size(-1) == 1:
+                        observation_noise = observation_noise.squeeze(-1)
+                    mvn = self.likelihood(mvn, X, noise=observation_noise)
+                else:
+                    mvn = self.likelihood(mvn, X)
+        posterior = GPyTorchPosterior(mvn=mvn)
+        if hasattr(self, "outcome_transform"):
+            posterior = self.outcome_transform.untransform_posterior(posterior)
+        return posterior
 
-    def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> "Model":
+    def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> Model:
         r"""Condition the model on new observations.
 
         Args:
@@ -146,17 +167,22 @@ class GPyTorchModel(Model, ABC):
             >>> new_Y = torch.sin(new_X[:, 0]) + torch.cos(new_X[:, 1])
             >>> model = model.condition_on_observations(X=new_X, Y=new_Y)
         """
-        noise = kwargs.get("noise", None)
+        Yvar = kwargs.get("noise", None)
+        if hasattr(self, "outcome_transform"):
+            # pass the transformed data to get_fantasy_model below
+            # (unless we've already trasnformed if BatchedMultiOutputGPyTorchModel)
+            if not isinstance(self, BatchedMultiOutputGPyTorchModel):
+                Y, Yvar = self.outcome_transform(Y, Yvar)
         # validate using strict=False, since we cannot tell if Y has an explicit
         # output dimension
-        self._validate_tensor_args(X=X, Y=Y, Yvar=noise, strict=False)
-        if Y.shape[-1] == 1:
-            targets = Y.squeeze(-1)
-            if noise is not None:
-                kwargs.update({"noise": noise.squeeze(-1)})
-        else:
-            targets = Y
-        return self.get_fantasy_model(inputs=X, targets=targets, **kwargs)
+        self._validate_tensor_args(X=X, Y=Y, Yvar=Yvar, strict=False)
+        if Y.size(-1) == 1:
+            Y = Y.squeeze(-1)
+            if Yvar is not None:
+                kwargs.update({"noise": Yvar.squeeze(-1)})
+        # get_fantasy_model will properly copy any existing outcome transforms
+        # (since it deepcopies the original model)
+        return self.get_fantasy_model(inputs=X, targets=Y, **kwargs)
 
 
 class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
@@ -170,6 +196,31 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
     _input_batch_shape: torch.Size
     _aug_batch_shape: torch.Size
 
+    @staticmethod
+    def get_batch_dimensions(
+        train_X: Tensor, train_Y: Tensor
+    ) -> Tuple[torch.Size, torch.Size]:
+        r"""Get the raw batch shape and output-augmented batch shape of the inputs.
+
+        Args:
+            train_X: A `n x d` or `batch_shape x n x d` (batch mode) tensor of training
+                features.
+            train_Y: A `n x m` or `batch_shape x n x m` (batch mode) tensor of
+                training observations.
+
+        Returns:
+            2-element tuple containing
+
+            - The `input_batch_shape`
+            - The output-augmented batch shape: `input_batch_shape x (m)`
+        """
+        input_batch_shape = train_X.shape[:-2]
+        aug_batch_shape = input_batch_shape
+        num_outputs = train_Y.shape[-1]
+        if num_outputs > 1:
+            aug_batch_shape += torch.Size([num_outputs])
+        return input_batch_shape, aug_batch_shape
+
     def _set_dimensions(self, train_X: Tensor, train_Y: Tensor) -> None:
         r"""Store the number of outputs and the batch shape.
 
@@ -180,10 +231,9 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
                 training observations.
         """
         self._num_outputs = train_Y.shape[-1]
-        self._input_batch_shape = train_X.shape[:-2]
-        self._aug_batch_shape = self._input_batch_shape
-        if self._num_outputs > 1:
-            self._aug_batch_shape += torch.Size([self._num_outputs])
+        self._input_batch_shape, self._aug_batch_shape = self.get_batch_dimensions(
+            train_X=train_X, train_Y=train_Y
+        )
 
     def _transform_tensor_args(
         self, X: Tensor, Y: Tensor, Yvar: Optional[Tensor] = None
@@ -219,42 +269,44 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
         self,
         X: Tensor,
         output_indices: Optional[List[int]] = None,
-        observation_noise: bool = False,
+        observation_noise: Union[bool, Tensor] = False,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
 
         Args:
-            X: A `(batch_shape) x q x d`-dim Tensor, where `d` is the dimension of the
-                feature space and `q` is the number of points considered jointly.
+            X: A `(batch_shape) x q x d`-dim Tensor, where `d` is the dimension
+                of the feature space and `q` is the number of points considered
+                jointly.
             output_indices: A list of indices, corresponding to the outputs over
                 which to compute the posterior (if the model is multi-output).
                 Can be used to speed up computation if only a subset of the
                 model's outputs are required for optimization. If omitted,
                 computes the posterior over all model outputs.
-            observation_noise: If True, add observation noise to the posterior.
+            observation_noise: If True, add the observation noise from the
+                likelihood to the posterior. If a Tensor, use it directly as the
+                observation noise (must be of shape `(batch_shape) x q x m`).
 
         Returns:
             A `GPyTorchPosterior` object, representing `batch_shape` joint
             distributions over `q` points and the outputs selected by
-            `output_indices` each. Includes observation noise if
-            `observation_noise=True`.
+            `output_indices` each. Includes observation noise if specified.
         """
         self.eval()  # make sure model is in eval mode
-        with ExitStack() as es:
-            es.enter_context(gpt_settings.debug(False))
-            es.enter_context(gpt_settings.fast_pred_var())
-            es.enter_context(
-                gpt_settings.detach_test_caches(settings.propagate_grads.off())
-            )
+        with gpt_posterior_settings():
             # insert a dimension for the output dimension
             if self._num_outputs > 1:
                 X, output_dim_idx = add_output_dim(
                     X=X, original_batch_shape=self._input_batch_shape
                 )
             mvn = self(X)
-            if observation_noise:
-                if isinstance(self.likelihood, FixedNoiseGaussianLikelihood):
+            if observation_noise is not False:
+                if torch.is_tensor(observation_noise):
+                    # TODO: Validate noise shape
+                    # make observation_noise `batch_shape x q x n`
+                    obs_noise = observation_noise.transpose(-1, -2)
+                    mvn = self.likelihood(mvn, X, noise=obs_noise)
+                elif isinstance(self.likelihood, FixedNoiseGaussianLikelihood):
                     # Use the mean of the previous noise values (TODO: be smarter here).
                     noise = self.likelihood.noise.mean().expand(X.shape[:-1])
                     mvn = self.likelihood(mvn, X, noise=noise)
@@ -272,11 +324,15 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
                     for t in output_indices
                 ]
                 mvn = MultitaskMultivariateNormal.from_independent_mvns(mvns=mvns)
-        return GPyTorchPosterior(mvn=mvn)
+
+        posterior = GPyTorchPosterior(mvn=mvn)
+        if hasattr(self, "outcome_transform"):
+            posterior = self.outcome_transform.untransform_posterior(posterior)
+        return posterior
 
     def condition_on_observations(
         self, X: Tensor, Y: Tensor, **kwargs: Any
-    ) -> "BatchedMultiOutputGPyTorchModel":
+    ) -> BatchedMultiOutputGPyTorchModel:
         r"""Condition the model on new observations.
 
         Args:
@@ -298,7 +354,6 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
             conditioned on the new observations `(X, Y)` (and possibly noise
             observations passed in via kwargs).
 
-
         Example:
             >>> train_X = torch.rand(20, 2)
             >>> train_Y = torch.cat(
@@ -310,6 +365,9 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
             >>> model = model.condition_on_observations(X=new_X, Y=new_Y)
         """
         noise = kwargs.get("noise")
+        if hasattr(self, "outcome_transform"):
+            # we need to apply transforms before shifting batch indices around
+            Y, noise = self.outcome_transform(Y, noise)
         self._validate_tensor_args(X=X, Y=Y, Yvar=noise, strict=False)
         inputs = X
         if self._num_outputs > 1:
@@ -333,6 +391,50 @@ class BatchedMultiOutputGPyTorchModel(GPyTorchModel):
         fantasy_model._aug_batch_shape = fantasy_model.train_targets.shape[:-1]
         return fantasy_model
 
+    def subset_output(self, idcs: List[int]) -> BatchedMultiOutputGPyTorchModel:
+        r"""Subset the model along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the model to.
+
+        Returns:
+            The current model, subset to the specified output indices.
+        """
+        try:
+            subset_batch_dict = self._subset_batch_dict
+        except AttributeError:
+            raise NotImplementedError(
+                "subset_output requires the model to define a `_subset_dict` attribute"
+            )
+
+        m = len(idcs)
+        tidxr = torch.tensor(idcs)
+        idxr = tidxr if m > 1 else idcs[0]
+        new_tail_bs = torch.Size([m]) if m > 1 else torch.Size()
+        new_model = deepcopy(self)
+
+        new_model._num_outputs = m
+        new_model._aug_batch_shape = new_model._aug_batch_shape[:-1] + new_tail_bs
+        new_model.train_inputs = tuple(
+            ti[..., idxr, :, :] for ti in new_model.train_inputs
+        )
+        new_model.train_targets = new_model.train_targets[..., idxr, :]
+
+        # adjust batch shapes of parameters/buffers if necessary
+        for full_name, p in itertools.chain(
+            new_model.named_parameters(), new_model.named_buffers()
+        ):
+            if full_name in subset_batch_dict:
+                idx = subset_batch_dict[full_name]
+                new_data = p.index_select(idx, tidxr)
+                if m == 1:
+                    new_data = new_data.squeeze(idx)
+                p.data = new_data
+            mod_name = full_name.split(".")[:-1]
+            mod_batch_shape(new_model, mod_name, m if m > 1 else 0)
+
+        return new_model
+
 
 class ModelListGPyTorchModel(GPyTorchModel, ABC):
     r"""Abstract base class for models based on multi-output GPyTorch models.
@@ -341,16 +443,11 @@ class ModelListGPyTorchModel(GPyTorchModel, ABC):
     evaluation of submodels.
     """
 
-    @abstractproperty
-    def num_outputs(self) -> int:
-        r"""The number of outputs of the model."""
-        pass  # pragma: no cover
-
     def posterior(
         self,
         X: Tensor,
         output_indices: Optional[List[int]] = None,
-        observation_noise: bool = False,
+        observation_noise: Union[bool, Tensor] = False,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
@@ -364,39 +461,62 @@ class ModelListGPyTorchModel(GPyTorchModel, ABC):
                 Can be used to speed up computation if only a subset of the
                 model's outputs are required for optimization. If omitted,
                 computes the posterior over all model outputs.
-            observation_noise: If True, add observation noise to the posterior.
+            observation_noise: If True, add the observation noise from the
+                respective likelihoods to the posterior. If a Tensor of shape
+                `(batch_shape) x q x m`, use it directly as the observation
+                noise (with `observation_noise[...,i]` added to the posterior
+                of the `i`-th model).
 
         Returns:
             A `GPyTorchPosterior` object, representing `batch_shape` joint
             distributions over `q` points and the outputs selected by
             `output_indices` each. Includes measurement noise if
-            `observation_noise=True`.
+            `observation_noise` is specified.
         """
         self.eval()  # make sure model is in eval mode
-        with ExitStack() as es:
-            es.enter_context(gpt_settings.debug(False))
-            es.enter_context(gpt_settings.fast_pred_var())
-            es.enter_context(
-                gpt_settings.detach_test_caches(settings.propagate_grads.off())
-            )
+        mvn_gen: Iterator
+        with gpt_posterior_settings():
+            # only compute what's necessary
             if output_indices is not None:
                 mvns = [self.forward_i(i, X) for i in output_indices]
-                if observation_noise:
-                    lh_kwargs = [
-                        {"noise": lh.noise.mean().expand(X.shape[:-1])}
-                        if isinstance(lh, FixedNoiseGaussianLikelihood)
-                        else {}
-                        for lh in self.likelihood.likelihoods
-                    ]
+                if observation_noise is not False:
+                    if torch.is_tensor(observation_noise):
+                        lh_kwargs = [
+                            {"noise": observation_noise[..., i]}
+                            for i, lh in enumerate(self.likelihood.likelihoods)
+                        ]
+                    else:
+                        lh_kwargs = [
+                            {"noise": lh.noise.mean().expand(X.shape[:-1])}
+                            if isinstance(lh, FixedNoiseGaussianLikelihood)
+                            else {}
+                            for lh in self.likelihood.likelihoods
+                        ]
                     mvns = [
                         self.likelihood_i(i, mvn, X, **lkws)
                         for i, mvn, lkws in zip(output_indices, mvns, lh_kwargs)
                     ]
+                mvn_gen = zip(output_indices, mvns)
             else:
                 mvns = self(*[X for _ in range(self.num_outputs)])
-                if observation_noise:
-                    # TODO: Allow passing in observation noise via kwarg
-                    mvns = self.likelihood(*[(mvn, X) for mvn in mvns])
+                if observation_noise is not False:
+                    if torch.is_tensor(observation_noise):
+                        mvns = self.likelihood(
+                            *[(mvn, X) for mvn in mvns], noise=observation_noise
+                        )
+                    else:
+                        mvns = self.likelihood(*[(mvn, X) for mvn in mvns])
+                mvn_gen = enumerate(mvns)
+        # apply output transforms of individual models if present
+        mvns = []
+        for i, mvn in mvn_gen:
+            try:
+                oct = self.models[i].outcome_transform
+                tf_mvn = oct.untransform_posterior(GPyTorchPosterior(mvn)).mvn
+            except AttributeError:
+                tf_mvn = mvn
+            mvns.append(tf_mvn)
+        # return result as a GPyTorchPosteriors
         if len(mvns) == 1:
             return GPyTorchPosterior(mvn=mvns[0])
         else:
@@ -406,10 +526,10 @@ class ModelListGPyTorchModel(GPyTorchModel, ABC):
 
     def condition_on_observations(
         self, X: Tensor, Y: Tensor, **kwargs: Any
-    ) -> "ModelListGPyTorchModel":
+    ) -> ModelListGPyTorchModel:
+        class_name = self.__class__.__name__
         raise NotImplementedError(
-            "`condition_on_observations` not implemented in "
-            "`ModelListGPyTorchModel` base class"
+            f"`condition_on_observations` not implemented in {class_name}"
         )
 
 
@@ -424,7 +544,7 @@ class MultiTaskGPyTorchModel(GPyTorchModel, ABC):
         self,
         X: Tensor,
         output_indices: Optional[List[int]] = None,
-        observation_noise: bool = False,
+        observation_noise: Union[bool, Tensor] = False,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         r"""Computes the posterior over model outputs at the provided points.
@@ -438,33 +558,36 @@ class MultiTaskGPyTorchModel(GPyTorchModel, ABC):
                 Can be used to speed up computation if only a subset of the
                 model's outputs are required for optimization. If omitted,
                 computes the posterior over all model outputs.
-            observation_noise: If True, add observation noise to the posterior.
+            observation_noise: If True, add observation noise from the respective
+                likelihoods. If a Tensor, specifies the observation noise levels
+                to add.
 
         Returns:
             A `GPyTorchPosterior` object, representing `batch_shape` joint
             distributions over `q` points and the outputs selected by
             `output_indices`. Includes measurement noise if
-            `observation_noise=True`.
+            `observation_noise` is specified.
         """
         if output_indices is None:
             output_indices = self._output_tasks
         if any(i not in self._output_tasks for i in output_indices):
             raise ValueError("Too many output indices")
+        cls_name = self.__class__.__name__
+        if hasattr(self, "outcome_transform"):
+            raise NotImplementedError(
+                f"Outcome transforms currently not supported by {cls_name}"
+            )
 
         # construct evaluation X
         X_full = _make_X_full(X=X, output_indices=output_indices, tf=self._task_feature)
 
         self.eval()  # make sure model is in eval mode
-        with ExitStack() as es:
-            es.enter_context(gpt_settings.debug(False))
-            es.enter_context(gpt_settings.fast_pred_var())
-            es.enter_context(
-                gpt_settings.detach_test_caches(settings.propagate_grads.off())
-            )
+        with gpt_posterior_settings():
             mvn = self(X_full)
-            if observation_noise:
-                # TODO: Allow passing in observation noise via kwarg
-                mvn = self.likelihood(mvn, X_full)
+            if observation_noise is not False:
+                raise NotImplementedError(
+                    f"Specifying observation noise is not yet supported by {cls_name}"
+                )
         # If single-output, return the posterior of a single-output model
         if len(output_indices) == 1:
             return GPyTorchPosterior(mvn=mvn)
