@@ -1,11 +1,15 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Abstract base module for all BoTorch models.
 """
 
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 from typing import Any, List, Optional
 
@@ -47,7 +51,25 @@ class Model(Module, ABC):
         """
         pass  # pragma: no cover
 
-    def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> "Model":
+    @property
+    def num_outputs(self) -> int:
+        r"""The number of outputs of the model."""
+        cls_name = self.__class__.__name__
+        raise NotImplementedError(f"{cls_name} does not define num_outputs property")
+
+    def subset_output(self, idcs: List[int]) -> Model:
+        r"""Subset the model along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the model to.
+
+        Returns:
+            A `Model` object of the same type and with the same parameters as
+            the current model, subset to the specified output indices.
+        """
+        raise NotImplementedError
+
+    def condition_on_observations(self, X: Tensor, Y: Tensor, **kwargs: Any) -> Model:
         r"""Condition the model on new observations.
 
         Args:
@@ -76,7 +98,7 @@ class Model(Module, ABC):
         sampler: MCSampler,
         observation_noise: bool = True,
         **kwargs: Any,
-    ) -> "Model":
+    ) -> Model:
         r"""Construct a fantasy model.
 
         Constructs a fantasy model in the following fashion:
