@@ -1,12 +1,17 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Model List GP Regression models.
 """
 
-from typing import Any
+from __future__ import annotations
+
+from copy import deepcopy
+from typing import Any, List
 
 from gpytorch.models import IndependentModelList
 from torch import Tensor
@@ -34,6 +39,8 @@ class ModelListGP(IndependentModelList, ModelListGPyTorchModel):
 
         Args:
             *gp_models: An variable number of single-output BoTorch models.
+                If models have input/output transforms, these are honored
+                individually for each model.
 
         Example:
             >>> model1 = SingleTaskGP(train_X1, train_Y1)
@@ -44,7 +51,7 @@ class ModelListGP(IndependentModelList, ModelListGPyTorchModel):
 
     def condition_on_observations(
         self, X: Tensor, Y: Tensor, **kwargs: Any
-    ) -> "ModelListGP":
+    ) -> ModelListGP:
         r"""Condition the model on new observations.
 
         Args:
@@ -85,3 +92,14 @@ class ModelListGP(IndependentModelList, ModelListGPyTorchModel):
         else:
             kwargs_ = kwargs
         return super().get_fantasy_model(inputs, targets, **kwargs_)
+
+    def subset_output(self, idcs: List[int]) -> ModelListGP:
+        r"""Subset the model along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the model to.
+
+        Returns:
+            The current model, subset to the specified output indices.
+        """
+        return self.__class__(*[deepcopy(self.models[i]) for i in idcs])
