@@ -1,15 +1,20 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Utiltiy functions for models.
 """
 
+from __future__ import annotations
+
 import warnings
 from typing import List, Optional, Tuple
 
 import torch
+from gpytorch.module import Module
 from gpytorch.utils.broadcasting import _mul_broadcast_shape
 from torch import Tensor
 
@@ -36,7 +41,7 @@ def _make_X_full(X: Tensor, output_indices: List[int], tf: int) -> Tensor:
     )
     X_l, X_r = X[..., :tf], X[..., tf:]
     return torch.cat(
-        [torch.cat([X_l, indexer, X_r], dim=-1) for indexer in indexers], dim=0
+        [torch.cat([X_l, indexer, X_r], dim=-1) for indexer in indexers], dim=-2
     )
 
 
@@ -87,14 +92,15 @@ def add_output_dim(X: Tensor, original_batch_shape: torch.Size) -> Tuple[Tensor,
     of the training inputs, but can also include extra batch dimensions.
 
     Args:
-        X: A `(new_batch_shape) x (original_batch_shape) x n x d` tensor of features.
+        X: A `(new_batch_shape) x (original_batch_shape) x n x d` tensor of
+            features.
         original_batch_shape: the batch shape of the model's training inputs.
 
     Returns:
         2-element tuple containing
 
         - A `(new_batch_shape) x (original_batch_shape) x m x n x d` tensor of
-        features.
+            features.
         - The index corresponding to the output dimension.
     """
     X_batch_shape = X.shape[:-2]
@@ -219,3 +225,25 @@ def validate_input_scaling(
             raise InputDataError("Input data contains negative variances.")
     check_min_max_scaling(X=train_X, raise_on_fail=raise_on_fail)
     check_standardization(Y=train_Y, raise_on_fail=raise_on_fail)
+
+
+def mod_batch_shape(module: Module, names: List[str], b: int) -> None:
+    r"""Recursive helper to modify gpytorch modules' batch shape attribute.
+
+    Modifies the module in-place.
+
+    Args:
+        module: The module to be modified.
+        names: The list of names to access the attribute. If the full name of
+            the module is `"module.sub_module.leaf_module"`, this will be
+            `["sub_module", "leaf_module"]`.
+        b: The new size of the last element of the module's `batch_shape`
+            attribute.
+    """
+    if len(names) == 0:
+        return
+    m = getattr(module, names[0])
+    if len(names) == 1 and hasattr(m, "batch_shape") and len(m.batch_shape) > 0:
+        m.batch_shape = m.batch_shape[:-1] + torch.Size([b] if b > 0 else [])
+    else:
+        mod_batch_shape(module=m, names=names[1:], b=b)
