@@ -1,15 +1,12 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 from .initializers import initialize_q_batch, initialize_q_batch_nonneg
 from .numpy_converter import module_to_array, set_params_with_array
-from .optimize import (
-    gen_batch_initial_conditions,
-    joint_optimize,
-    optimize_acqf,
-    sequential_optimize,
-)
+from .optimize import gen_batch_initial_conditions, optimize_acqf, optimize_acqf_cyclic
 
 
 __all__ = [
@@ -17,8 +14,7 @@ __all__ = [
     "initialize_q_batch",
     "initialize_q_batch_nonneg",
     "optimize_acqf",
-    "joint_optimize",
+    "optimize_acqf_cyclic",
     "module_to_array",
-    "sequential_optimize",
     "set_params_with_array",
 ]
