@@ -1,29 +1,47 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Tools for model fitting.
 """
 
+from __future__ import annotations
+
 import time
 import warnings
-from collections import OrderedDict
-from typing import Any, Dict, List, NamedTuple, Optional, Tuple, Union
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, Set, Tuple, Union
 
 import numpy as np
+from gpytorch import settings as gpt_settings
 from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
 from scipy.optimize import Bounds, minimize
 from torch import Tensor
+from torch.nn import Module
 from torch.optim.adam import Adam
 from torch.optim.optimizer import Optimizer
 
 from ..exceptions.warnings import OptimizationWarning
 from .numpy_converter import TorchAttr, module_to_array, set_params_with_array
-from .utils import ConvergenceCriterion, _filter_kwargs, _get_extra_mll_args
+from .utils import (
+    ConvergenceCriterion,
+    _filter_kwargs,
+    _get_extra_mll_args,
+    _scipy_objective_and_grad,
+)
 
 
 ParameterBounds = Dict[str, Tuple[Optional[float], Optional[float]]]
+TScipyObjective = Callable[
+    [np.ndarray, MarginalLogLikelihood, Dict[str, TorchAttr]], Tuple[float, np.ndarray]
+]
+TModToArray = Callable[
+    [Module, Optional[ParameterBounds], Optional[Set[str]]],
+    Tuple[np.ndarray, Dict[str, TorchAttr], Optional[np.ndarray]],
+]
+TArrayToMod = Callable[[Module, np.ndarray, Dict[str, TorchAttr]], Module]
 
 
 class OptimizationIteration(NamedTuple):
@@ -38,6 +56,7 @@ def fit_gpytorch_torch(
     optimizer_cls: Optimizer = Adam,
     options: Optional[Dict[str, Any]] = None,
     track_iterations: bool = True,
+    approx_mll: bool = True,
 ) -> Tuple[MarginalLogLikelihood, Dict[str, Union[float, List[OptimizationIteration]]]]:
     r"""Fit a gpytorch model by maximizing MLL with a torch optimizer.
 
@@ -57,6 +76,10 @@ def fit_gpytorch_torch(
             to specify the maximum number of iterations.
         track_iterations: Track the function values and wall time for each
             iteration.
+        approx_mll: If True, use gpytorch's approximate MLL computation (
+            according to the gpytorch defaults based on the training at size).
+            Unlike for the deterministic algorithms used in fit_gpytorch_scipy,
+            this is not an issue for stochastic optimizers.
 
     Returns:
         2-element tuple containing
@@ -114,11 +137,12 @@ def fit_gpytorch_torch(
     train_inputs, train_targets = mll.model.train_inputs, mll.model.train_targets
     while not converged:
         optimizer.zero_grad()
-        output = mll.model(*train_inputs)
-        # we sum here to support batch mode
-        args = [output, train_targets] + _get_extra_mll_args(mll)
-        loss = -mll(*args).sum()
-        loss.backward()
+        with gpt_settings.fast_computations(log_prob=approx_mll):
+            output = mll.model(*train_inputs)
+            # we sum here to support batch mode
+            args = [output, train_targets] + _get_extra_mll_args(mll)
+            loss = -mll(*args).sum()
+            loss.backward()
         loss_trajectory.append(loss.item())
         for name, param in mll.named_parameters():
             param_trajectory[name].append(param.detach().clone())
@@ -150,11 +174,15 @@ def fit_gpytorch_scipy(
     method: str = "L-BFGS-B",
     options: Optional[Dict[str, Any]] = None,
     track_iterations: bool = True,
+    approx_mll: bool = False,
+    scipy_objective: TScipyObjective = _scipy_objective_and_grad,
+    module_to_array_func: TModToArray = module_to_array,
+    module_from_array_func: TArrayToMod = set_params_with_array,
 ) -> Tuple[MarginalLogLikelihood, Dict[str, Union[float, List[OptimizationIteration]]]]:
     r"""Fit a gpytorch model by maximizing MLL with a scipy optimizer.
 
     The model and likelihood in mll must already be in train mode.
-    Note: this method requires that the model has `train_inputs` and `train_targets`.
+    This method requires that the model has `train_inputs` and `train_targets`.
 
     Args:
         mll: MarginalLogLikelihood to be maximized.
@@ -164,6 +192,10 @@ def fit_gpytorch_scipy(
         options: Dictionary of solver options, passed along to scipy.minimize.
         track_iterations: Track the function values and wall time for each
             iteration.
+        approx_mll: If True, use gpytorch's approximate MLL computation. This is
+            disabled by default since the stochasticity is an issue for
+            determistic optimizers). Enabling this is only recommended when
+            working with large training data sets (n>2000).
 
     Returns:
         2-element tuple containing
@@ -182,7 +214,7 @@ def fit_gpytorch_scipy(
         >>> mll.eval()
     """
     options = options or {}
-    x0, property_dict, bounds = module_to_array(
+    x0, property_dict, bounds = module_to_array_func(
         module=mll, bounds=bounds, exclude=options.pop("exclude", None)
     )
     x0 = x0.astype(np.float64)
@@ -199,23 +231,22 @@ def fit_gpytorch_scipy(
 
     cb = store_iteration if track_iterations else None
 
-    res = minimize(
-        _scipy_objective_and_grad,
-        x0,
-        args=(mll, property_dict),
-        bounds=bounds,
-        method=method,
-        jac=True,
-        options=options,
-        callback=cb,
-    )
-    iterations = []
-    if track_iterations:
-        for i, xk in enumerate(xs):
-            obj, _ = _scipy_objective_and_grad(
-                x=xk, mll=mll, property_dict=property_dict
-            )
-            iterations.append(OptimizationIteration(i, obj, ts[i]))
+    with gpt_settings.fast_computations(log_prob=approx_mll):
+        res = minimize(
+            scipy_objective,
+            x0,
+            args=(mll, property_dict),
+            bounds=bounds,
+            method=method,
+            jac=True,
+            options=options,
+            callback=cb,
+        )
+        iterations = []
+        if track_iterations:
+            for i, xk in enumerate(xs):
+                obj, _ = scipy_objective(x=xk, mll=mll, property_dict=property_dict)
+                iterations.append(OptimizationIteration(i, obj, ts[i]))
     # Construct info dict
     info_dict = {
         "fopt": float(res.fun),
@@ -233,48 +264,5 @@ def fit_gpytorch_scipy(
             f"Fitting failed with the optimizer reporting '{msg}'", OptimizationWarning
         )
     # Set to optimum
-    mll = set_params_with_array(mll, res.x, property_dict)
+    mll = module_from_array_func(mll, res.x, property_dict)
     return mll, info_dict
-
-
-def _scipy_objective_and_grad(
-    x: np.ndarray, mll: MarginalLogLikelihood, property_dict: Dict[str, TorchAttr]
-) -> Tuple[float, np.ndarray]:
-    r"""Get objective and gradient in format that scipy expects.
-
-    Args:
-        x: The (flattened) input parameters.
-        mll: The MarginalLogLikelihood module to evaluate.
-        property_dict: The property dictionary required to "unflatten" the input
-            parameter vector, as generated by `module_to_array`.
-
-    Returns:
-        2-element tuple containing
-
-        - The objective value.
-        - The gradient of the objective.
-    """
-    mll = set_params_with_array(mll, x, property_dict)
-    train_inputs, train_targets = mll.model.train_inputs, mll.model.train_targets
-    mll.zero_grad()
-    try:  # catch linear algebra errors in gpytorch
-        output = mll.model(*train_inputs)
-        args = [output, train_targets] + _get_extra_mll_args(mll)
-        loss = -mll(*args).sum()
-    except RuntimeError as e:
-        if "singular" in e.args[0]:
-            return float("nan"), np.full_like(x, "nan")
-        else:
-            raise e  # pragma: nocover
-    loss.backward()
-    param_dict = OrderedDict(mll.named_parameters())
-    grad = []
-    for p_name in property_dict:
-        t = param_dict[p_name].grad
-        if t is None:
-            # this deals with parameters that do not affect the loss
-            grad.append(np.zeros(property_dict[p_name].shape.numel()))
-        else:
-            grad.append(t.detach().view(-1).cpu().double().clone().numpy())
-    mll.zero_grad()
-    return loss.item(), np.concatenate(grad)
