@@ -1,25 +1,32 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Utilities for optimization.
 """
 
-import warnings
-from inspect import signature
-from typing import Any, Callable, Dict, List, Optional, Union
+from __future__ import annotations
 
+import warnings
+from collections import OrderedDict
+from inspect import signature
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+
+import numpy as np
 import torch
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
 from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
 from gpytorch.mlls.sum_marginal_log_likelihood import SumMarginalLogLikelihood
-from gpytorch.mlls.variational_elbo import VariationalELBO
+from gpytorch.utils.errors import NanError
 from torch import Tensor
 
 from ..exceptions.errors import BotorchError
 from ..exceptions.warnings import BotorchWarning
 from ..models.gpytorch import GPyTorchModel
+from .numpy_converter import TorchAttr, set_params_with_array
 
 
 def sample_all_priors(model: GPyTorchModel) -> None:
@@ -211,7 +218,7 @@ def _expand_bounds(
 
 
 def _get_extra_mll_args(
-    mll: MarginalLogLikelihood
+    mll: MarginalLogLikelihood,
 ) -> Union[List[Tensor], List[List[Tensor]]]:
     r"""Obtain extra arguments for MarginalLogLikelihood objects.
 
@@ -223,18 +230,59 @@ def _get_extra_mll_args(
 
     Returns:
         Extra arguments for the MarginalLogLikelihood.
+        Returns an empty list if the mll type is unknown.
     """
     if isinstance(mll, ExactMarginalLogLikelihood):
         return list(mll.model.train_inputs)
     elif isinstance(mll, SumMarginalLogLikelihood):
         return [list(x) for x in mll.model.train_inputs]
-    elif isinstance(mll, VariationalELBO):
-        return []
-    else:
-        raise ValueError("Do not know how to optimize MLL type.")
+    return []
 
 
 def _filter_kwargs(function: Callable, **kwargs: Any) -> Any:
     r"""Filter out kwargs that are not applicable for a given function.
     Return a copy of given kwargs dict with only the required kwargs."""
     return {k: v for k, v in kwargs.items() if k in signature(function).parameters}
+
+
+def _scipy_objective_and_grad(
+    x: np.ndarray, mll: MarginalLogLikelihood, property_dict: Dict[str, TorchAttr]
+) -> Tuple[float, np.ndarray]:
+    r"""Get objective and gradient in format that scipy expects.
+
+    Args:
+        x: The (flattened) input parameters.
+        mll: The MarginalLogLikelihood module to evaluate.
+        property_dict: The property dictionary required to "unflatten" the input
+            parameter vector, as generated by `module_to_array`.
+
+    Returns:
+        2-element tuple containing
+
+        - The objective value.
+        - The gradient of the objective.
+    """
+    mll = set_params_with_array(mll, x, property_dict)
+    train_inputs, train_targets = mll.model.train_inputs, mll.model.train_targets
+    mll.zero_grad()
+    try:  # catch linear algebra errors in gpytorch
+        output = mll.model(*train_inputs)
+        args = [output, train_targets] + _get_extra_mll_args(mll)
+        loss = -mll(*args).sum()
+    except RuntimeError as e:
+        if isinstance(e, NanError) or "singular" in e.args[0]:
+            return float("nan"), np.full_like(x, "nan")
+        else:
+            raise e  # pragma: nocover
+    loss.backward()
+    param_dict = OrderedDict(mll.named_parameters())
+    grad = []
+    for p_name in property_dict:
+        t = param_dict[p_name].grad
+        if t is None:
+            # this deals with parameters that do not affect the loss
+            grad.append(np.zeros(property_dict[p_name].shape.numel()))
+        else:
+            grad.append(t.detach().view(-1).cpu().double().clone().numpy())
+    mll.zero_grad()
+    return loss.item(), np.concatenate(grad)
