@@ -1,11 +1,15 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Synthetic functions for multi-fidelity optimization benchmarks.
 """
 
+from __future__ import annotations
+
 import math
 from typing import Optional
 
