@@ -1,12 +1,16 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Synthetic functions for optimization benchmarks.
 Reference: https://www.sfu.ca/~ssurjano/optimization.html
 """
 
+from __future__ import annotations
+
 import math
 from abc import ABC, abstractmethod
 from typing import List, Optional, Tuple
