@@ -1,17 +1,25 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Some basic data transformation helpers.
 """
 
+from __future__ import annotations
+
+from contextlib import ExitStack, contextmanager
 from functools import wraps
-from typing import Any, Callable, Optional
+from typing import Any, Callable, List, Optional
 
 import torch
+from gpytorch import settings as gpt_settings
 from torch import Tensor
 
+from .. import settings
+
 
 def squeeze_last_dim(Y: Tensor) -> Tensor:
     r"""Squeeze the last dimension of a Tensor.
@@ -96,6 +104,30 @@ def unnormalize(X: Tensor, bounds: Tensor) -> Tensor:
     return X * (bounds[1] - bounds[0]) + bounds[0]
 
 
+def normalize_indices(indices: Optional[List[int]], d: int) -> Optional[List[int]]:
+    r"""Normalize a list of indices to ensure that they are positive.
+
+    Args:
+        indices: A list of indices (may contain negative indices for indexing
+            "from the back").
+        d: The dimension of the tensor to index.
+
+    Returns:
+        A normalized list of indices such that each index is between `0` and
+        `d-1`, or None if indices is None.
+    """
+    if indices is None:
+        return indices
+    normalized_indices = []
+    for i in indices:
+        if i < 0:
+            i = i + d
+        if i < 0 or i > d - 1:
+            raise ValueError(f"Index {i} out of bounds for tensor or length {d}.")
+        normalized_indices.append(i)
+    return normalized_indices
+
+
 def t_batch_mode_transform(
     expected_q: Optional[int] = None,
 ) -> Callable[[Callable[[Any, Tensor], Any]], Callable[[Any, Tensor], Any]]:
@@ -201,3 +233,15 @@ def match_batch_shape(X: Tensor, Y: Tensor) -> Tensor:
 def convert_to_target_pre_hook(module, *args):
     r"""Pre-hook for automatically calling `.to(X)` on module prior to `forward`"""
     module.to(args[0][0])
+
+
+@contextmanager
+def gpt_posterior_settings():
+    r"""Context manager for settings used for computing model posteriors."""
+    with ExitStack() as es:
+        es.enter_context(gpt_settings.debug(False))
+        es.enter_context(gpt_settings.fast_pred_var())
+        es.enter_context(
+            gpt_settings.detach_test_caches(settings.propagate_grads.off())
+        )
+        yield
