@@ -1,6 +1,8 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 import math
 
@@ -36,10 +38,21 @@ NEI_NOISE = [
 ]
 
 
+class DummyAnalyticAcquisitionFunction(AnalyticAcquisitionFunction):
+    def forward(self, X):
+        pass
+
+
 class TestAnalyticAcquisitionFunction(BotorchTestCase):
     def test_abstract_raises(self):
         with self.assertRaises(TypeError):
             AnalyticAcquisitionFunction()
+        # raise if model is multi-output, but no objective is given
+        mean = torch.zeros(1, 2)
+        variance = torch.ones(1, 2)
+        mm = MockModel(MockPosterior(mean=mean, variance=variance))
+        with self.assertRaises(UnsupportedError):
+            DummyAnalyticAcquisitionFunction(model=mm)
 
 
 class TestExpectedImprovement(BotorchTestCase):
@@ -111,9 +124,8 @@ class TestExpectedImprovement(BotorchTestCase):
             mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             variance2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
-            module2 = ExpectedImprovement(model=mm2, best_f=0.0)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                ExpectedImprovement(model=mm2, best_f=0.0)
 
             # test objective (single-output)
             mean = torch.tensor([[[0.5]], [[0.25]]], device=self.device, dtype=dtype)
@@ -170,9 +182,8 @@ class TestPosteriorMean(BotorchTestCase):
             # check for proper error if multi-output model
             mean2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2))
-            module2 = PosteriorMean(model=mm2)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                PosteriorMean(model=mm2)
 
     def test_posterior_mean_batch(self):
         for dtype in (torch.float, torch.double):
@@ -187,9 +198,8 @@ class TestPosteriorMean(BotorchTestCase):
             # check for proper error if multi-output model
             mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2))
-            module2 = PosteriorMean(model=mm2)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                PosteriorMean(model=mm2)
 
 
 class TestProbabilityOfImprovement(BotorchTestCase):
@@ -215,9 +225,8 @@ class TestProbabilityOfImprovement(BotorchTestCase):
             mean2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             variance2 = torch.ones_like(mean2)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
-            module2 = ProbabilityOfImprovement(model=mm2, best_f=0.0)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                ProbabilityOfImprovement(model=mm2, best_f=0.0)
 
     def test_probability_of_improvement_batch(self):
         for dtype in (torch.float, torch.double):
@@ -235,9 +244,8 @@ class TestProbabilityOfImprovement(BotorchTestCase):
             mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             variance2 = torch.ones_like(mean2)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
-            module2 = ProbabilityOfImprovement(model=mm2, best_f=0.0)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                ProbabilityOfImprovement(model=mm2, best_f=0.0)
 
 
 class TestUpperConfidenceBound(BotorchTestCase):
@@ -263,9 +271,8 @@ class TestUpperConfidenceBound(BotorchTestCase):
             mean2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             variance2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
-            module2 = UpperConfidenceBound(model=mm2, beta=1.0)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                UpperConfidenceBound(model=mm2, beta=1.0)
 
     def test_upper_confidence_bound_batch(self):
         for dtype in (torch.float, torch.double):
@@ -285,9 +292,8 @@ class TestUpperConfidenceBound(BotorchTestCase):
             mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             variance2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
-            module2 = UpperConfidenceBound(model=mm2, beta=1.0)
             with self.assertRaises(UnsupportedError):
-                module2(X)
+                UpperConfidenceBound(model=mm2, beta=1.0)
 
 
 class TestConstrainedExpectedImprovement(BotorchTestCase):
@@ -452,7 +458,7 @@ class TestNoisyExpectedImprovement(BotorchTestCase):
             self.assertLess(val[1].item(), 1e-6)
             # test gradient
             val.sum().backward()
-            self.assertGreater(X_test.grad[0].abs().item(), 1e-4)
+            self.assertGreater(X_test.grad[0].abs().item(), 1e-5)
             # test without gradient
             with torch.no_grad():
                 nEI(X_test)
