@@ -1,12 +1,23 @@
 #!/usr/bin/env python3
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
-
+import itertools
 from unittest import mock
 
 import torch
-from botorch.acquisition import monte_carlo, utils
-from botorch.acquisition.objective import MCAcquisitionObjective
+from botorch.acquisition import monte_carlo
+from botorch.acquisition.objective import GenericMCObjective, MCAcquisitionObjective
+from botorch.acquisition.utils import (
+    expand_trace_observations,
+    get_acquisition_function,
+    get_infeasible_cost,
+    project_to_target_fidelity,
+    prune_inferior_points,
+)
+from botorch.exceptions import UnsupportedError
 from botorch.sampling.samplers import IIDNormalSampler, SobolQMCNormalSampler
 from botorch.utils.testing import BotorchTestCase, MockModel, MockPosterior
 from torch import Tensor
@@ -30,7 +41,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
 
     @mock.patch(f"{monte_carlo.__name__}.qExpectedImprovement")
     def test_GetQEI(self, mock_acqf):
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qEI",
             model=self.model,
             objective=self.objective,
@@ -59,7 +70,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
     @mock.patch(f"{monte_carlo.__name__}.qProbabilityOfImprovement")
     def test_GetQPI(self, mock_acqf):
         # basic test
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qPI",
             model=self.model,
             objective=self.objective,
@@ -86,7 +97,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         self.assertEqual(sampler.seed, 1)
         self.assertTrue(torch.equal(kwargs["X_pending"], self.X_pending))
         # test with different tau, non-qmc
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qPI",
             model=self.model,
             objective=self.objective,
@@ -110,7 +121,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
     @mock.patch(f"{monte_carlo.__name__}.qNoisyExpectedImprovement")
     def test_GetQNEI(self, mock_acqf):
         # basic test
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qNEI",
             model=self.model,
             objective=self.objective,
@@ -130,7 +141,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         self.assertEqual(sampler.sample_shape, torch.Size([self.mc_samples]))
         self.assertEqual(sampler.seed, 1)
         # test with non-qmc, no X_pending
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qNEI",
             model=self.model,
             objective=self.objective,
@@ -154,7 +165,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
     @mock.patch(f"{monte_carlo.__name__}.qSimpleRegret")
     def test_GetQSR(self, mock_acqf):
         # basic test
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qSR",
             model=self.model,
             objective=self.objective,
@@ -178,7 +189,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         self.assertEqual(sampler.seed, 1)
         self.assertTrue(torch.equal(kwargs["X_pending"], self.X_pending))
         # test with non-qmc
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qSR",
             model=self.model,
             objective=self.objective,
@@ -201,7 +212,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
     def test_GetQUCB(self, mock_acqf):
         # make sure beta is specified
         with self.assertRaises(ValueError):
-            acqf = utils.get_acquisition_function(
+            acqf = get_acquisition_function(
                 acquisition_function_name="qUCB",
                 model=self.model,
                 objective=self.objective,
@@ -210,7 +221,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
                 mc_samples=self.mc_samples,
                 seed=self.seed,
             )
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qUCB",
             model=self.model,
             objective=self.objective,
@@ -236,7 +247,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         self.assertEqual(sampler.seed, 1)
         self.assertTrue(torch.equal(kwargs["X_pending"], self.X_pending))
         # test with different tau, non-qmc
-        acqf = utils.get_acquisition_function(
+        acqf = get_acquisition_function(
             acquisition_function_name="qUCB",
             model=self.model,
             objective=self.objective,
@@ -259,7 +270,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
 
     def test_GetUnknownAcquisitionFunction(self):
         with self.assertRaises(NotImplementedError):
-            utils.get_acquisition_function(
+            get_acquisition_function(
                 acquisition_function_name="foo",
                 model=self.model,
                 objective=self.objective,
@@ -283,7 +294,138 @@ class TestGetInfeasibleCost(BotorchTestCase):
             mm = MockModel(MockPosterior(mean=means, variance=variances))
             # means - 6 * std = [-0.8, -1, -0.6, 1, 3.2]. After applying the
             # objective, the minimum becomes -6.0, so 6.0 should be returned.
-            M = utils.get_infeasible_cost(
+            M = get_infeasible_cost(
                 X=X, model=mm, objective=lambda Y: Y.squeeze(-1) - 5.0
             )
             self.assertEqual(M, 6.0)
+
+
+class TestPruneInferiorPoints(BotorchTestCase):
+    def test_prune_inferior_points(self):
+        for dtype in (torch.float, torch.double):
+            X = torch.rand(3, 2, device=self.device, dtype=dtype)
+            # the event shape is `q x t` = 3 x 1
+            samples = torch.tensor(
+                [[-1.0], [0.0], [1.0]], device=self.device, dtype=dtype
+            )
+            mm = MockModel(MockPosterior(samples=samples))
+            # test that a batched X raises errors
+            with self.assertRaises(UnsupportedError):
+                prune_inferior_points(model=mm, X=X.expand(2, 3, 2))
+            # test that a batched model raises errors (event shape is `q x t` = 3 x 1)
+            mm2 = MockModel(MockPosterior(samples=samples.expand(2, 3, 1)))
+            with self.assertRaises(UnsupportedError):
+                prune_inferior_points(model=mm2, X=X)
+            # test that invalid max_frac is checked properly
+            with self.assertRaises(ValueError):
+                prune_inferior_points(model=mm, X=X, max_frac=1.1)
+            # test basic behaviour
+            X_pruned = prune_inferior_points(model=mm, X=X)
+            self.assertTrue(torch.equal(X_pruned, X[[-1]]))
+            # test custom objective
+            neg_id_obj = GenericMCObjective(lambda X: -X.squeeze(-1))
+            X_pruned = prune_inferior_points(model=mm, X=X, objective=neg_id_obj)
+            self.assertTrue(torch.equal(X_pruned, X[[0]]))
+            # test non-repeated samples (requires mocking out MockPosterior's rsample)
+            samples = torch.tensor(
+                [[[3.0], [0.0], [0.0]], [[0.0], [2.0], [0.0]], [[0.0], [0.0], [1.0]]],
+                device=self.device,
+                dtype=dtype,
+            )
+            with mock.patch.object(MockPosterior, "rsample", return_value=samples):
+                mm = MockModel(MockPosterior(samples=samples))
+                X_pruned = prune_inferior_points(model=mm, X=X)
+            self.assertTrue(torch.equal(X_pruned, X))
+            # test max_frac limiting
+            with mock.patch.object(MockPosterior, "rsample", return_value=samples):
+                mm = MockModel(MockPosterior(samples=samples))
+                X_pruned = prune_inferior_points(model=mm, X=X, max_frac=2 / 3)
+            self.assertTrue(torch.equal(X_pruned, X[:2]))
+            # test that zero-probability is in fact pruned
+            samples[2, 0, 0] = 10
+            with mock.patch.object(MockPosterior, "rsample", return_value=samples):
+                mm = MockModel(MockPosterior(samples=samples))
+                X_pruned = prune_inferior_points(model=mm, X=X)
+            self.assertTrue(torch.equal(X_pruned, X[:2]))
+
+
+class TestFidelityUtils(BotorchTestCase):
+    def test_project_to_target_fidelity(self):
+        for batch_shape, dtype in itertools.product(
+            ([], [2]), (torch.float, torch.double)
+        ):
+            X = torch.rand(*batch_shape, 3, 4, device=self.device, dtype=dtype)
+            # test default behavior
+            X_proj = project_to_target_fidelity(X)
+            ones = torch.ones(*X.shape[:-1], 1, device=self.device, dtype=dtype)
+            self.assertTrue(torch.equal(X_proj[..., :, [-1]], ones))
+            self.assertTrue(torch.equal(X_proj[..., :-1], X[..., :-1]))
+            # test custom target fidelity
+            target_fids = {2: 0.5}
+            X_proj = project_to_target_fidelity(X, target_fidelities=target_fids)
+            self.assertTrue(torch.equal(X_proj[..., :, [2]], 0.5 * ones))
+            # test multiple target fidelities
+            target_fids = {2: 0.5, 0: 0.1}
+            X_proj = project_to_target_fidelity(X, target_fidelities=target_fids)
+            self.assertTrue(torch.equal(X_proj[..., :, [0]], 0.1 * ones))
+            self.assertTrue(torch.equal(X_proj[..., :, [2]], 0.5 * ones))
+            # test gradients
+            X.requires_grad_(True)
+            X_proj = project_to_target_fidelity(X, target_fidelities=target_fids)
+            out = (X_proj ** 2).sum()
+            out.backward()
+            self.assertTrue(torch.all(X.grad[..., [0, 2]] == 0))
+            self.assertTrue(torch.equal(X.grad[..., [1, 3]], 2 * X[..., [1, 3]]))
+
+    def test_expand_trace_observations(self):
+        for batch_shape, dtype in itertools.product(
+            ([], [2]), (torch.float, torch.double)
+        ):
+            q, d = 3, 4
+            X = torch.rand(*batch_shape, q, d, device=self.device, dtype=dtype)
+            # test nullop behavior
+            self.assertTrue(torch.equal(expand_trace_observations(X), X))
+            self.assertTrue(
+                torch.equal(expand_trace_observations(X, fidelity_dims=[1]), X)
+            )
+            # test default behavior
+            num_tr = 2
+            X_expanded = expand_trace_observations(X, num_trace_obs=num_tr)
+            self.assertEqual(
+                X_expanded.shape, torch.Size(batch_shape + [q * (1 + num_tr), d])
+            )
+            for i in range(num_tr):
+                X_sub = X_expanded[..., q * i : q * (i + 1), :]
+                self.assertTrue(torch.equal(X_sub[..., :-1], X[..., :-1]))
+                X_sub_expected = (1 - i / (num_tr + 1)) * X[..., :q, -1]
+                self.assertTrue(torch.equal(X_sub[..., -1], X_sub_expected))
+            # test custom fidelity dims
+            fdims = [0, 2]
+            num_tr = 3
+            X_expanded = expand_trace_observations(
+                X, fidelity_dims=fdims, num_trace_obs=num_tr
+            )
+            self.assertEqual(
+                X_expanded.shape, torch.Size(batch_shape + [q * (1 + num_tr), d])
+            )
+            for j, i in itertools.product([1, 3], range(num_tr)):
+                X_sub = X_expanded[..., q * i : q * (i + 1), j]
+                self.assertTrue(torch.equal(X_sub, X[..., j]))
+            for j, i in itertools.product(fdims, range(num_tr)):
+                X_sub = X_expanded[..., q * i : q * (i + 1), j]
+                X_sub_expected = (1 - i / (1 + num_tr)) * X[..., :q, j]
+                self.assertTrue(torch.equal(X_sub, X_sub_expected))
+            # test gradients
+            num_tr = 2
+            fdims = [1]
+            X.requires_grad_(True)
+            X_expanded = expand_trace_observations(
+                X, fidelity_dims=fdims, num_trace_obs=num_tr
+            )
+            out = X_expanded.sum()
+            out.backward()
+            grad_exp = torch.full_like(X, 1 + num_tr)
+            grad_exp[..., fdims] = 1 + sum(
+                (i + 1) / (num_tr + 1) for i in range(num_tr)
+            )
+            self.assertTrue(torch.allclose(X.grad, grad_exp))
