@@ -1,6 +1,8 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 from botorch.models.model import Model
 from botorch.utils.testing import BotorchTestCase
@@ -20,3 +22,7 @@ class TestBaseModel(BotorchTestCase):
         model = NotSoAbstractBaseModel()
         with self.assertRaises(NotImplementedError):
             model.condition_on_observations(None, None)
+        with self.assertRaises(NotImplementedError):
+            model.num_outputs
+        with self.assertRaises(NotImplementedError):
+            model.subset_output([0])
