@@ -1,8 +1,10 @@
 #! /usr/bin/env python3
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
-
-import math
+import itertools
 import warnings
 
 import torch
@@ -11,8 +13,9 @@ from botorch.exceptions.warnings import OptimizationWarning
 from botorch.fit import fit_gpytorch_model
 from botorch.models import ModelListGP
 from botorch.models.gp_regression import FixedNoiseGP, SingleTaskGP
+from botorch.models.transforms import Standardize
 from botorch.posteriors import GPyTorchPosterior
-from botorch.utils.testing import BotorchTestCase
+from botorch.utils.testing import BotorchTestCase, _get_random_data
 from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
 from gpytorch.kernels import MaternKernel, ScaleKernel
 from gpytorch.likelihoods import LikelihoodList
@@ -22,44 +25,47 @@ from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikeliho
 from gpytorch.priors import GammaPrior
 
 
-def _get_random_data(n, **tkwargs):
-    train_x1 = torch.linspace(0, 0.95, n + 1, **tkwargs).unsqueeze(
-        -1
-    ) + 0.05 * torch.rand(n + 1, 1, **tkwargs)
-    train_x2 = torch.linspace(0, 0.95, n, **tkwargs).unsqueeze(-1) + 0.05 * torch.rand(
-        n, 1, **tkwargs
+def _get_model(n, fixed_noise=False, use_octf=False, **tkwargs):
+    train_x1, train_y1 = _get_random_data(
+        batch_shape=torch.Size(), num_outputs=1, n=10, **tkwargs
     )
-    train_y1 = torch.sin(train_x1 * (2 * math.pi)) + 0.2 * torch.randn_like(train_x1)
-    train_y2 = torch.cos(train_x2 * (2 * math.pi)) + 0.2 * torch.randn_like(train_x2)
-    return train_x1, train_x2, train_y1, train_y2
-
-
-def _get_model(n, fixed_noise=False, **tkwargs):
-    train_x1, train_x2, train_y1, train_y2 = _get_random_data(n=n, **tkwargs)
+    train_x2, train_y2 = _get_random_data(
+        batch_shape=torch.Size(), num_outputs=1, n=11, **tkwargs
+    )
+    octfs = [Standardize(m=1), Standardize(m=1)] if use_octf else [None, None]
     if fixed_noise:
         train_y1_var = 0.1 + 0.1 * torch.rand_like(train_y1, **tkwargs)
         train_y2_var = 0.1 + 0.1 * torch.rand_like(train_y2, **tkwargs)
         model1 = FixedNoiseGP(
-            train_X=train_x1, train_Y=train_y1, train_Yvar=train_y1_var
+            train_X=train_x1,
+            train_Y=train_y1,
+            train_Yvar=train_y1_var,
+            outcome_transform=octfs[0],
         )
         model2 = FixedNoiseGP(
-            train_X=train_x2, train_Y=train_y2, train_Yvar=train_y2_var
+            train_X=train_x2,
+            train_Y=train_y2,
+            train_Yvar=train_y2_var,
+            outcome_transform=octfs[1],
         )
     else:
-        model1 = SingleTaskGP(train_X=train_x1, train_Y=train_y1)
-        model2 = SingleTaskGP(train_X=train_x2, train_Y=train_y2)
+        model1 = SingleTaskGP(
+            train_X=train_x1, train_Y=train_y1, outcome_transform=octfs[0]
+        )
+        model2 = SingleTaskGP(
+            train_X=train_x2, train_Y=train_y2, outcome_transform=octfs[1]
+        )
     model = ModelListGP(model1, model2)
     return model.to(**tkwargs)
 
 
 class TestModelListGP(BotorchTestCase):
     def test_ModelListGP(self):
-        for double in (False, True):
-            tkwargs = {
-                "device": self.device,
-                "dtype": torch.double if double else torch.float,
-            }
-            model = _get_model(n=10, **tkwargs)
+        for dtype, use_octf in itertools.product(
+            (torch.float, torch.double), (False, True)
+        ):
+            tkwargs = {"device": self.device, "dtype": dtype}
+            model = _get_model(n=10, use_octf=use_octf, **tkwargs)
             self.assertIsInstance(model, ModelListGP)
             self.assertIsInstance(model.likelihood, LikelihoodList)
             for m in model.models:
@@ -68,6 +74,8 @@ class TestModelListGP(BotorchTestCase):
                 matern_kernel = m.covar_module.base_kernel
                 self.assertIsInstance(matern_kernel, MaternKernel)
                 self.assertIsInstance(matern_kernel.lengthscale_prior, GammaPrior)
+                if use_octf:
+                    self.assertIsInstance(m.outcome_transform, Standardize)
 
             # test constructing likelihood wrapper
             mll = SumMarginalLogLikelihood(model.likelihood, model)
@@ -85,11 +93,30 @@ class TestModelListGP(BotorchTestCase):
                     mll, options={"maxiter": 1}, max_retries=1, sequential=False
                 )
 
+            # test subset outputs
+            subset_model = model.subset_output([1])
+            self.assertIsInstance(subset_model, ModelListGP)
+            self.assertEqual(len(subset_model.models), 1)
+            sd_subset = subset_model.models[0].state_dict()
+            sd = model.models[1].state_dict()
+            self.assertTrue(set(sd_subset.keys()) == set(sd.keys()))
+            self.assertTrue(all(torch.equal(v, sd[k]) for k, v in sd_subset.items()))
+
             # test posterior
             test_x = torch.tensor([[0.25], [0.75]], **tkwargs)
             posterior = model.posterior(test_x)
             self.assertIsInstance(posterior, GPyTorchPosterior)
             self.assertIsInstance(posterior.mvn, MultitaskMultivariateNormal)
+            if use_octf:
+                # ensure un-transformation is applied
+                submodel = model.models[0]
+                p0 = submodel.posterior(test_x)
+                tmp_tf = submodel.outcome_transform
+                del submodel.outcome_transform
+                p0_tf = submodel.posterior(test_x)
+                submodel.outcome_transform = tmp_tf
+                expected_var = tmp_tf.untransform_posterior(p0_tf).variance
+                self.assertTrue(torch.allclose(p0.variance, expected_var))
 
             # test observation_noise
             posterior = model.posterior(test_x, observation_noise=True)
@@ -126,12 +153,11 @@ class TestModelListGP(BotorchTestCase):
                 model.condition_on_observations(f_x, torch.rand(3, 2, 3, **tkwargs))
 
     def test_ModelListGP_fixed_noise(self):
-        for double in (False, True):
-            tkwargs = {
-                "device": self.device,
-                "dtype": torch.double if double else torch.float,
-            }
-            model = _get_model(n=10, fixed_noise=True, **tkwargs)
+        for dtype, use_octf in itertools.product(
+            (torch.float, torch.double), (False, True)
+        ):
+            tkwargs = {"device": self.device, "dtype": dtype}
+            model = _get_model(n=10, fixed_noise=True, use_octf=use_octf, **tkwargs)
             self.assertIsInstance(model, ModelListGP)
             self.assertIsInstance(model.likelihood, LikelihoodList)
             for m in model.models:
@@ -154,14 +180,23 @@ class TestModelListGP(BotorchTestCase):
             posterior = model.posterior(test_x)
             self.assertIsInstance(posterior, GPyTorchPosterior)
             self.assertIsInstance(posterior.mvn, MultitaskMultivariateNormal)
+            if use_octf:
+                # ensure un-transformation is applied
+                submodel = model.models[0]
+                p0 = submodel.posterior(test_x)
+                tmp_tf = submodel.outcome_transform
+                del submodel.outcome_transform
+                p0_tf = submodel.posterior(test_x)
+                submodel.outcome_transform = tmp_tf
+                expected_var = tmp_tf.untransform_posterior(p0_tf).variance
+                self.assertTrue(torch.allclose(p0.variance, expected_var))
 
-            # TODO: Add test back in once gpytorch > 0.3.5 is released
-            # # test output_indices
-            # posterior = model.posterior(
-            #     test_x, output_indices=[0], observation_noise=True
-            # )
-            # self.assertIsInstance(posterior, GPyTorchPosterior)
-            # self.assertIsInstance(posterior.mvn, MultivariateNormal)
+            # test output_indices
+            posterior = model.posterior(
+                test_x, output_indices=[0], observation_noise=True
+            )
+            self.assertIsInstance(posterior, GPyTorchPosterior)
+            self.assertIsInstance(posterior.mvn, MultivariateNormal)
 
             # test condition_on_observations
             f_x = torch.rand(2, 1, **tkwargs)
@@ -198,7 +233,12 @@ class TestModelListGP(BotorchTestCase):
 
     def test_ModelListGP_single(self):
         tkwargs = {"device": self.device, "dtype": torch.float}
-        train_x1, train_x2, train_y1, train_y2 = _get_random_data(n=10, **tkwargs)
+        train_x1, train_y1 = _get_random_data(
+            batch_shape=torch.Size(), num_outputs=1, n=10, **tkwargs
+        )
+        train_x2, train_y2 = _get_random_data(
+            batch_shape=torch.Size(), num_outputs=1, n=11, **tkwargs
+        )
         model1 = SingleTaskGP(train_X=train_x1, train_Y=train_y1)
         model = ModelListGP(model1)
         model.to(**tkwargs)
