@@ -1,6 +1,8 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 import math
 import warnings
@@ -29,7 +31,7 @@ def _get_random_mt_data(**tkwargs):
     full_train_i = torch.cat([train_i_task1, train_i_task2])
     full_train_y = torch.cat([train_y1, train_y2])
     train_X = torch.stack([full_train_x, full_train_i.type_as(full_train_x)], dim=-1)
-    train_Y = full_train_y
+    train_Y = full_train_y.unsqueeze(-1)  # add output dim
     return train_X, train_Y
 
 
@@ -63,13 +65,11 @@ def _get_fixed_noise_model_single_output(**tkwargs):
 
 class TestMultiTaskGP(BotorchTestCase):
     def test_MultiTaskGP(self):
-        for double in (False, True):
-            tkwargs = {
-                "device": self.device,
-                "dtype": torch.double if double else torch.float,
-            }
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
             model = _get_model(**tkwargs)
             self.assertIsInstance(model, MultiTaskGP)
+            self.assertEqual(model.num_outputs, 2)
             self.assertIsInstance(model.likelihood, GaussianLikelihood)
             self.assertIsInstance(model.mean_module, ConstantMean)
             self.assertIsInstance(model.covar_module, ScaleKernel)
@@ -96,12 +96,11 @@ class TestMultiTaskGP(BotorchTestCase):
             self.assertEqual(posterior_f.mean.shape, torch.Size([2, 2]))
             self.assertEqual(posterior_f.variance.shape, torch.Size([2, 2]))
 
-            # test posterior w/ observation noise
-            posterior_o = model.posterior(test_x, observation_noise=True)
-            self.assertIsInstance(posterior_o, GPyTorchPosterior)
-            self.assertIsInstance(posterior_f.mvn, MultitaskMultivariateNormal)
-            self.assertEqual(posterior_f.mean.shape, torch.Size([2, 2]))
-            self.assertEqual(posterior_f.variance.shape, torch.Size([2, 2]))
+            # test that posterior w/ observation noise raises appropriate error
+            with self.assertRaises(NotImplementedError):
+                model.posterior(test_x, observation_noise=True)
+            with self.assertRaises(NotImplementedError):
+                model.posterior(test_x, observation_noise=torch.rand(2, **tkwargs))
 
             # test posterior w/ single output index
             posterior_f = model.posterior(test_x, output_indices=[0])
@@ -122,7 +121,7 @@ class TestMultiTaskGP(BotorchTestCase):
 
             # test that unsupported batch shape MTGPs throw correct error
             with self.assertRaises(ValueError):
-                MultiTaskGP(torch.rand(2, 2, 2), torch.rand(2, 1), 0)
+                MultiTaskGP(torch.rand(2, 2, 2), torch.rand(2, 2, 1), 0)
 
             # test that bad feature index throws correct error
             train_X, train_Y = _get_random_mt_data(**tkwargs)
@@ -133,14 +132,17 @@ class TestMultiTaskGP(BotorchTestCase):
             with self.assertRaises(RuntimeError):
                 MultiTaskGP(train_X, train_Y, 0, output_tasks=[2])
 
+            # test error if outcome_transform attribute is present
+            model.outcome_transform = None
+            with self.assertRaises(NotImplementedError):
+                model.posterior(test_x)
+
     def test_MultiTaskGP_single_output(self):
-        for double in (False, True):
-            tkwargs = {
-                "device": self.device,
-                "dtype": torch.double if double else torch.float,
-            }
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
             model = _get_model_single_output(**tkwargs)
             self.assertIsInstance(model, MultiTaskGP)
+            self.assertEqual(model.num_outputs, 1)
             self.assertIsInstance(model.likelihood, GaussianLikelihood)
             self.assertIsInstance(model.mean_module, ConstantMean)
             self.assertIsInstance(model.covar_module, ScaleKernel)
@@ -174,13 +176,11 @@ class TestMultiTaskGP(BotorchTestCase):
 
 class TestFixedNoiseMultiTaskGP(BotorchTestCase):
     def test_FixedNoiseMultiTaskGP(self):
-        for double in (False, True):
-            tkwargs = {
-                "device": self.device,
-                "dtype": torch.double if double else torch.float,
-            }
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
             model = _get_fixed_noise_model(**tkwargs)
             self.assertIsInstance(model, FixedNoiseMultiTaskGP)
+            self.assertEqual(model.num_outputs, 2)
             self.assertIsInstance(model.likelihood, FixedNoiseGaussianLikelihood)
             self.assertIsInstance(model.mean_module, ConstantMean)
             self.assertIsInstance(model.covar_module, ScaleKernel)
@@ -207,7 +207,11 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
             self.assertEqual(posterior_f.mean.shape, torch.Size([2, 2]))
             self.assertEqual(posterior_f.variance.shape, torch.Size([2, 2]))
 
-            # TODO: test posterior w/ observation noise
+            # test that posterior w/ observation noise raises appropriate error
+            with self.assertRaises(NotImplementedError):
+                model.posterior(test_x, observation_noise=True)
+            with self.assertRaises(NotImplementedError):
+                model.posterior(test_x, observation_noise=torch.rand(2, **tkwargs))
 
             # test posterior w/ single output index
             posterior_f = model.posterior(test_x, output_indices=[0])
@@ -229,7 +233,7 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
             # test that unsupported batch shape MTGPs throw correct error
             with self.assertRaises(ValueError):
                 FixedNoiseMultiTaskGP(
-                    torch.rand(2, 2, 2), torch.rand(2, 1), torch.rand(2, 1), 0
+                    torch.rand(2, 2, 2), torch.rand(2, 2, 1), torch.rand(2, 2, 1), 0
                 )
 
             # test that bad feature index throws correct error
@@ -243,13 +247,11 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
                 FixedNoiseMultiTaskGP(train_X, train_Y, train_Yvar, 0, output_tasks=[2])
 
     def test_FixedNoiseMultiTaskGP_single_output(self):
-        for double in (False, True):
-            tkwargs = {
-                "device": self.device,
-                "dtype": torch.double if double else torch.float,
-            }
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
             model = _get_fixed_noise_model_single_output(**tkwargs)
             self.assertIsInstance(model, FixedNoiseMultiTaskGP)
+            self.assertEqual(model.num_outputs, 1)
             self.assertIsInstance(model.likelihood, FixedNoiseGaussianLikelihood)
             self.assertIsInstance(model.mean_module, ConstantMean)
             self.assertIsInstance(model.covar_module, ScaleKernel)
