@@ -1,6 +1,8 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 import warnings
 from copy import deepcopy
@@ -23,7 +25,6 @@ from gpytorch.kernels.scale_kernel import ScaleKernel
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
 from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
 from gpytorch.mlls.sum_marginal_log_likelihood import SumMarginalLogLikelihood
-from gpytorch.mlls.variational_elbo import VariationalELBO
 from gpytorch.priors.smoothed_box_prior import SmoothedBoxPrior
 from gpytorch.priors.torch_priors import GammaPrior
 
@@ -162,22 +163,18 @@ class TestFixFeatures(BotorchTestCase):
         )
 
 
-class testGetExtraMllArgs(BotorchTestCase):
+class TestGetExtraMllArgs(BotorchTestCase):
     def test_get_extra_mll_args(self):
         train_X = torch.rand(3, 5)
         train_Y = torch.rand(3, 1)
         model = SingleTaskGP(train_X=train_X, train_Y=train_Y)
+
         # test ExactMarginalLogLikelihood
         exact_mll = ExactMarginalLogLikelihood(model.likelihood, model)
         exact_extra_args = _get_extra_mll_args(mll=exact_mll)
         self.assertEqual(len(exact_extra_args), 1)
         self.assertTrue(torch.equal(exact_extra_args[0], train_X))
 
-        # test VariationalELBO
-        elbo = VariationalELBO(model.likelihood, model, num_data=train_X.shape[0])
-        elbo_extra_args = _get_extra_mll_args(mll=elbo)
-        self.assertEqual(len(elbo_extra_args), 0)
-
         # test SumMarginalLogLikelihood
         model2 = ModelListGP(model)
         sum_mll = SumMarginalLogLikelihood(model2.likelihood, model2)
@@ -188,11 +185,11 @@ class testGetExtraMllArgs(BotorchTestCase):
 
         # test unsupported MarginalLogLikelihood type
         unsupported_mll = MarginalLogLikelihood(model.likelihood, model)
-        with self.assertRaises(ValueError):
-            _get_extra_mll_args(mll=unsupported_mll)
+        unsupported_mll_extra_args = _get_extra_mll_args(mll=unsupported_mll)
+        self.assertEqual(unsupported_mll_extra_args, [])
 
 
-class testExpandBounds(BotorchTestCase):
+class TestExpandBounds(BotorchTestCase):
     def test_expand_bounds(self):
         X = torch.zeros(2, 3)
         expected_bounds = torch.zeros(1, 3)
