@@ -1,6 +1,8 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Monolithic CUDA tests. This implements a single monolithic test for all
@@ -11,25 +13,26 @@ to the GPU running out of memory.
 """
 
 import unittest
+from pathlib import Path
 from typing import Union
-from unittest import TestCase, TestSuite
 
 import torch
 from botorch.utils.testing import BotorchTestCase
 
 
 @unittest.skipIf(not torch.cuda.is_available(), "CUDA not available")
-class TestBotorchCUDA(TestCase):
+class TestBotorchCUDA(unittest.TestCase):
     def test_cuda(self):
-        tests = unittest.TestLoader().discover(".")
+        test_dir = Path(__file__).parent.resolve()
+        tests = unittest.TestLoader().discover(test_dir)
         run_cuda_tests(tests)
 
 
-def run_cuda_tests(tests: Union[TestCase, TestSuite]) -> None:
+def run_cuda_tests(tests: Union[unittest.TestCase, unittest.TestSuite]) -> None:
     """Function for running all tests on cuda (except TestBotorchCUDA itself)"""
     if isinstance(tests, BotorchTestCase):
         tests.device = torch.device("cuda")
         tests.run()
-    elif isinstance(tests, TestSuite):
+    elif isinstance(tests, unittest.TestSuite):
         for tests_ in tests:
             run_cuda_tests(tests_)
