@@ -1,18 +1,24 @@
 #! /usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 import torch
+from botorch import settings
 from botorch.utils.testing import BotorchTestCase
 from botorch.utils.transforms import (
     concatenate_pending_points,
+    gpt_posterior_settings,
     match_batch_shape,
     normalize,
+    normalize_indices,
     squeeze_last_dim,
     standardize,
     t_batch_mode_transform,
     unnormalize,
 )
+from gpytorch import settings as gpt_settings
 from torch import Tensor
 
 
@@ -178,8 +184,38 @@ class TestMatchBatchShape(BotorchTestCase):
             match_batch_shape(X, Y)
 
 
+class TorchNormalizeIndices(BotorchTestCase):
+    def test_normalize_indices(self):
+        self.assertIsNone(normalize_indices(None, 3))
+        indices = [0, 2]
+        nlzd_indices = normalize_indices(indices, 3)
+        self.assertEqual(nlzd_indices, indices)
+        nlzd_indices = normalize_indices(indices, 4)
+        self.assertEqual(nlzd_indices, indices)
+        indices = [0, -1]
+        nlzd_indices = normalize_indices(indices, 3)
+        self.assertEqual(nlzd_indices, [0, 2])
+        with self.assertRaises(ValueError):
+            nlzd_indices = normalize_indices([3], 3)
+        with self.assertRaises(ValueError):
+            nlzd_indices = normalize_indices([-4], 3)
+
+
 class TestSqueezeLastDim(BotorchTestCase):
     def test_squeeze_last_dim(self):
         Y = torch.rand(2, 1, 1)
         Y_squeezed = squeeze_last_dim(Y=Y)
         self.assertTrue(torch.equal(Y_squeezed, Y.squeeze(-1)))
+
+
+class TestGPTPosteriorSettings(BotorchTestCase):
+    def test_gpt_posterior_settings(self):
+        for propagate_grads in (False, True):
+            with settings.propagate_grads(propagate_grads):
+                with gpt_posterior_settings():
+                    self.assertTrue(gpt_settings.debug.off())
+                    self.assertTrue(gpt_settings.fast_pred_var.on())
+                    if settings.propagate_grads.off():
+                        self.assertTrue(gpt_settings.detach_test_caches.on())
+                    else:
+                        self.assertTrue(gpt_settings.detach_test_caches.off())
